@@ -502,12 +502,16 @@ candidateDefnsC Args{..} nm f ps  =
   pats | maxPatternSize > 0  =  take maxPatternSize $ conjurePats es nm f
        | otherwise           =                        conjurePats es nm f
   fss  =  concatMapT ps2fss pats
-  es  =  map fst ps
+  -- replaces the any guard symbol with a guard of the correct type
+  es  =  [if isGuardSymbol e then conjureGuard f else e | (e,_) <- ps]
 
   eh  =  holeAsTypeOf efxs
   efxs  =  conjureVarApplication nm f
   (ef:_)  =  unfoldApp efxs
 
+  unguardT | any isGuardSymbol es  =  discardT isGuard
+           | otherwise             =  id
+
   keep | rewriting  =  isRootNormalC thy . fastMostGeneralVariation
        | otherwise  =  const True
 
@@ -517,13 +521,13 @@ candidateDefnsC Args{..} nm f ps  =
   appsWith :: Expr -> [Expr] -> [[Expr]]
   appsWith eh vs  =  enumerateAppsFor eh k $ vs ++ es
     where
-    k | atomicNumbers && isNumeric eh  =  \e -> keepNumeric e && keep e
-      | maxConstantSize > 0            =  \e -> keepConstant e && keep e
-      | otherwise                      =  keep
+    k e  =  keepNumeric e && keepConstant e && keep e
     -- discards non-atomic numeric ground expressions such as 1 + 1
-    keepNumeric e  =  isFun e || isConst e || not (isGround e)
+    keepNumeric | atomicNumbers && isNumeric eh  =  \e -> isFun e || isConst e || not (isGround e)
+                | otherwise                      =  const True
     -- discards big non-atomic ground expressions such as 1 + 1 or reverse [1,2]
-    keepConstant e  =  isFun e || isConst e || not (isGround e) || size e <= maxConstantSize
+    keepConstant | maxConstantSize > 0  =  \e -> isFun e || isConst e || not (isGround e) || size e <= maxConstantSize
+                 | otherwise            =  const True
 
   isRedundant | adHocRedundancy  =  \e -> isRedundantDefn e || isRedundantModuloRewriting (normalize thy) e
               | otherwise        =  const False
@@ -604,6 +608,7 @@ candidateDefnsC Args{..} nm f ps  =
   deconstructions  =  filter (conjureIsDeconstruction f maxTests)
                    $  concatMap candidateDeconstructionsFromHoled
                    $  concat . take maxDeconstructionSize
+                   $  unguardT
                    $  concatMapT (`appsWith` hs) [hs]
     where
     hs  =  nub $ conjureArgumentHoles f
@@ -612,7 +617,7 @@ candidateDefnsC Args{..} nm f ps  =
            .  discardT (\e -> e == ep)
            $  recsV' (tail (vars ep))
   recsV vs  =  filterT (\e -> any (`elem` vs) (vars e))
-            $  foldAppProducts ef [appsWith h vs | h <- conjureArgumentHoles f]
+            $  foldAppProducts ef [unguardT $ appsWith h vs | h <- conjureArgumentHoles f]
   -- like recs, but memoized
   recs' ep  =  fromMaybe errRP $ lookup ep eprs
     where
