@@ -19,6 +19,8 @@ module Conjure.Expr
   , compareSimplicity
   , ifFor
   , caseForOrd
+  , guardFor
+  , ifToGuard
   , valuesBFS
   , holesBFS
   , fillBFS
@@ -34,6 +36,9 @@ module Conjure.Expr
   , isZero
   , isNegative
   , isStrictSubexprOf
+  , isGuardSymbol
+  , isGuard
+  , hasGuard
 
   , enumerateAppsFor
   , enumerateFillings
@@ -203,6 +208,7 @@ apparentlyTerminates eRecursiveCall  =  at
 -- This currently works by checking if the function is an if, '&&' or '||'.
 mayNotEvaluateArgument :: Expr -> Bool
 mayNotEvaluateArgument (Value "if" ce :$ _ :$ _)  =  True
+mayNotEvaluateArgument (Value "|"  ce :$ _ :$ _)  =  True
 mayNotEvaluateArgument (Value "&&" ce :$ _)       =  True
 mayNotEvaluateArgument (Value "||" ce :$ _)       =  True
 mayNotEvaluateArgument _                          =  False
@@ -227,6 +233,27 @@ ifFor a  =  value "if" (\p x y -> if p then x else y `asTypeOf` a)
 caseForOrd :: Typeable a => a -> Expr
 caseForOrd a  =  value "case" (\o x y z -> case o of LT -> x; EQ -> y; GT -> z `asTypeOf` a)
 
+-- | Creates an if 'Expr' of the type argument of the given proxy.
+--
+-- The expression is named as @|@ as if to represent a guard.
+--
+-- > > guardFor (undefined :: Int)
+-- > (|) :: Bool -> Int -> Int -> Int
+--
+-- In the future, maybe we display these differently in Conjure as guards.
+--
+-- For now, this will signalize an if that can only appear as a root expression.
+guardFor :: Typeable a => a -> Expr
+guardFor a  =  value "|" (\p x y -> if p then x else y `asTypeOf` a)
+
+-- | Changes an if encoded as an 'Expr' to a guard encoded as an 'Expr'.
+--
+-- This works for both the dangling symbol, or for a full application.
+ifToGuard :: Expr -> Expr
+ifToGuard (Value "if" dyn)  =  Value "|" dyn
+ifToGuard (Value "if" dyn :$ ep :$ ex :$ ey)  =  Value "|" dyn :$ ep :$ ex :$ ey
+ifToGuard e  =  e
+
 -- | Lists terminal values in BFS order.
 --
 -- (cf. 'values', 'holesBFS', 'fillBFS')
@@ -346,12 +373,15 @@ enumerateAppsFor h keep es  =  for h
   for h  =  filter (\e -> typ h == typ e) es : apps
     where
     apps  =  foldr (\/) []
-          [  filterT keep $ fliproductWith (:$) (for hf) (for hx)
+          [  filterT keep $ fliproductWith (:$) (ufor hf) (ufor hx)
           |  hf <- hs
           ,  hx <- hs
           ,  Just hfx <- [hf $$ hx]
           ,  typ h == typ hfx
           ]
+  -- unguarded for
+  ufor | any isGuardSymbol es  =  filterT (not . isGuard) . for
+       | otherwise             =  for
 
 -- | Given an expression whose holes are /all of the same type/
 --   and an enumeration of 'Expr's of this same type,
@@ -386,6 +416,11 @@ recursiveToDynamic (efxs, ebody) n  =  fmap (\(_,_,d) -> d) . re (n * size ebody
     Nothing    -> Nothing
     Just (m,n,True)  -> re m n ex
     Just (m,n,False) -> re m n ey
+  -- the following repeats the above
+  re m n (Value "|" _ :$ ec :$ ex :$ ey)  =  case rev m n ec of
+    Nothing    -> Nothing
+    Just (m,n,True)  -> re m n ex
+    Just (m,n,False) -> re m n ey
   re m n (Value "||" _ :$ ep :$ eq)  =  case rev m n ep of
     Nothing        -> Nothing
     Just (m,n,True)  -> (m,n,) <$> toDynamic (val True)
@@ -553,6 +588,24 @@ isNegative :: Expr -> Bool
 isNegative (Value ('-':_) _)  =  True
 isNegative _  =  False
 
+isGuardSymbol :: Expr -> Bool
+isGuardSymbol (Value "|" _)  =  True
+isGuardSymbol _  =  False
+
+-- | Is the expression a guard application?
+isGuard :: Expr -> Bool
+isGuard (Value "|" _ :$ _ :$ _ :$ _)  =  True
+isGuard _  =  False
+
+-- | Has this expression a non-root guard?
+--
+-- This only checks the immediate sub-expressions,
+-- good enough for when enumerating.
+--
+-- Guards at the root are still allowed.
+hasGuard :: Expr -> Bool
+hasGuard (ef :$ ex)  =  isGuard ef || isGuard ex
+
 -- | Lists all variables in an expression
 --   that are of the same type of the expression itself.
 --
